<?php

namespace Drupal\status_dashboard\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\status_dashboard\ClientSiteInterface;

/**
 * Defines the client site entity class.
 *
 * @ContentEntityType(
 *   id = "client_site",
 *   label = @Translation("Client site"),
 *   label_collection = @Translation("Client sites"),
 *   label_singular = @Translation("Client site"),
 *   label_plural = @Translation("Client sites"),
 *   label_count = @PluralTranslation(
 *     singular = "@count client site",
 *     plural = "@count client sites",
 *   ),
 *   base_table = "status_dashboard_client_site",
 *   admin_permission = "administer status dashboard",
 *   entity_keys = {
 *     "id" = "id",
 *   },
 *   links = {
 *     "collection" = "/admin/content/client-site",
 *     "canonical" = "/admin/client-site/{client_site}",
 *   },
 *   field_ui_base_route = "entity.client_site.settings",
 * )
 */
class ClientSite extends ContentEntityBase implements ClientSiteInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public function hasSecurityUpdates() {
    return !$this->get('security_updates')->isEmpty();
  }

  /**
   * {@inheritdoc}
   */
  public function hasFeatureUpdates() {
    return !$this->get('feature_updates')->isEmpty();
  }

  /**
   * {@inheritdoc}
   */
  public function getSecurityUpdates() {
    return $this->get('security_updates')->first()->getValue();
  }

  /**
   * {@inheritdoc}
   */
  public function setSecurityUpdates(array $security_updates) {
    $this->set('security_updates', $security_updates);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getFeatureUpdates() {
    return $this->get('feature_updates')->first()->getValue();
  }

  /**
   * {@inheritdoc}
   */
  public function setFeatureUpdates(array $feature_updates) {
    $this->set('feature_updates', $feature_updates);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getSitename() {
    return $this->get('sitename')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setSitename(string $sitename) {
    $this->set('sitename', $sitename);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCoreStatus() {
    return $this->get('core_status')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCoreStatus(string $core_status) {
    $this->set('core_status', $core_status);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getModules() {
    return $this->get('modules')->getValue();
  }

  /**
   * {@inheritdoc}
   */
  public function setModules(array $modules) {
    $this->set('modules', $modules);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getLastChecked() {
    return $this->get('last_checked')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setLastChecked(int $last_checked) {
    $this->set('last_checked', $last_checked);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('ID'))
      ->setDescription(t('The ID of client web site.'))
      ->setReadOnly(TRUE);

    $fields['sitename'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Site name'))
      ->setDescription(t('Client web site name.'))
      ->setSettings([
        'default_value' => '',
        'max_length' => 100,
      ]);

    $fields['core_status'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Core status'))
      ->setDescription(t('Drupal version.'))
      ->setSettings([
        'default_value' => '',
        'max_length' => 100,
      ]);

    $fields['modules'] = BaseFieldDefinition::create('map')
      ->setLabel(t('Modules'))
      ->setDescription(t('Drupal modules version.'));

    $fields['security_updates'] = BaseFieldDefinition::create('map')
      ->setLabel(t('security updates'))
      ->setDescription(t('Drupal security updates.'));

    $fields['feature_updates'] = BaseFieldDefinition::create('map')
      ->setLabel(t('feature updates'))
      ->setDescription(t('Drupal feature updates.'));

    $fields['last_checked'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Last checked'))
      ->setDescription(t('The time that the Drupal version was checked.'))
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'timestamp',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 10,
      ])
      ->setDisplayConfigurable('form', TRUE);
    return $fields;
  }

}
