<?php

namespace Drupal\status_dashboard\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\status_dashboard\Entity\ClientSite;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Status dashboard form.
 */
class DashboardForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * DashboardForm constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'status_dashboard_dashboard';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attached']['library'][] = 'status_dashboard/status_dashboard';

    $form['dashboard'] = [
      '#type' => 'table',
      '#title' => $this->t('Dashboard'),
      '#header' => $this->getHeader(),
      '#sticky' => TRUE,
      '#required' => TRUE,
    ];

    $query = $this->entityTypeManager
      ->getStorage('client_site')->getQuery();
    $resultsIds = $query->execute();
    $results = $this->entityTypeManager
      ->getStorage('client_site')->loadMultiple($resultsIds);
    foreach ($results as $result) {
      $this->createRow($form, $result);
    }

    return $form;
  }

  /**
   * Creates a table row.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\status_dashboard\Entity\ClientSite $entity
   *   The row data.
   */
  private function createRow(array &$form, ClientSite $entity): void {
    $security_updates = $entity->hasSecurityUpdates() ? $entity->getSecurityUpdates() : NULL;
    $feature_updates = $entity->hasFeatureUpdates() ? $entity->getFeatureUpdates() : NULL;

    $statusColor = 'last-update';
    if (!empty($feature_updates['Drupal core'])) {
      $statusColor = 'available-feature-updates';
    }
    if (!empty($security_updates['Drupal core'])) {
      $statusColor = 'available-security-updates';
    }

    $core_status = [
      '#type' => 'html_tag',
      '#tag' => 'span',
      '#value' => $entity->getCoreStatus(),
      '#attributes' => [
        'class' => [$statusColor],
      ],
    ];

    $updates_list = [];

    if ($security_updates !== NULL) {
      $items = [];
      foreach ($security_updates as $key => $value) {
        $items[] = $key . ': ' . $value;
      }
      $updates_list[] = [
        '#type' => 'details',
        '#title' => $this->t('@amount security updates', ['@amount' => count($items)]),
        '#description' => [
          '#theme' => 'item_list',
          '#list_type' => 'ul',
          '#items' => $items,
        ],
        '#attributes' => ['class' => ['available-security-updates']],
        '#open' => TRUE,
      ];
    }
    else {
      $updates_list[] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('No security updates'),
        '#attributes' => ['class' => ['last-update']],
      ];
    }

    if ($feature_updates !== NULL) {
      $items = [];
      foreach ($feature_updates as $key => $value) {
        $items[] = $key . ': ' . $value;
      }
      $updates_list[] = [
        '#type' => 'details',
        '#title' => $this->t('@amount feature updates', ['@amount' => count($items)]),
        '#description' => [
          '#theme' => 'item_list',
          '#list_type' => 'ul',
          '#items' => $items,
        ],
        '#attributes' => ['class' => ['available-feature-updates']],
        '#open' => TRUE,
      ];
    }
    else {
      $updates_list[] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('No feature updates'),
        '#attributes' => ['class' => ['last-update']],
      ];
    }

    $form['dashboard']['#rows'][] = [
      ['data' => $entity->getSitename()],
      ['data' => $core_status],
      ['data' => $updates_list],
      ['data' => date('d-m-Y H:i', $entity->getLastChecked())],
    ];

  }

  /**
   * Gets the table header for the status dashboard.
   *
   * @return array[]
   *   Table header for the status dashboard.
   */
  public function getHeader(): array {
    return [
      $this->t('Sitename'),
      $this->t('Core status'),
      $this->t('Modules'),
      $this->t('Last checked'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Generic submit is not used.
  }

}
